/***************************************************************************
                          cataloghandler.cpp  -  description
                             -------------------
    begin                : Thu Mar 20 2003
    copyright            : (C) 2003 by Stefano Salvador
    email                : salva_ste@tin.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qdir.h>
#include <qfileinfo.h>

#include <klocale.h>
#include <kprogress.h>
#include <kmimetype.h>
#include <kmessagebox.h>

#include "cataloghandler.h"
#include "catalogentry.h"
#include "cattreeitem.h"

CatalogHandler::CatalogHandler()
{
  catl = 0;
}

CatalogHandler::~CatalogHandler()
{
  clear();
}

/** create a new catalogue a visualize it as a parent in ListView */
CatTreeItem* CatalogHandler::
newCatalog( QString dir, QString p, QString des, QDate* dat, QStringList* l, bool propagate )
{
  totEntries = 0;
  QDir d( dir );
  if( d.exists() )
  {
    prgDlg = new KProgressDialog( 0, "prgDlg", "Creating the Catalogue ...", "Creating the Catalogue ..." );
    prgDlg->progressBar()->setTotalSteps( 1000000 );
    prgDlg->show();
    totalEntries( d );
    prgDlg->progressBar()->setTotalSteps( totEntries + prgDlg->progressBar()->value() );
    CatTreeItem *ce = new CatTreeItem( 0,
      new CatalogEntry( p, 0, des, dat, l, CatalogEntry::E_CATALOG ) );

    if( propagate )
      catl = l;
    else
      catl = 0;

    if( scanDir( d, ce ) )
      KMessageBox::error( 0, i18n("Some file is NOT readable, catalogue is NOT complete.\n"
      "Check the read permissions in the folder specified") );

    delete prgDlg;
    logList.append( ce );
    return ce;
  }
  else
  {
    KMessageBox::error( 0, i18n("The mount point does not exist !") );
    return 0;
  }
}

CatTreeItem* CatalogHandler::
newCatalog( QString p, QString des, QDate* dat, QStringList* l )
{
    CatTreeItem *ce = new CatTreeItem( 0,
      new CatalogEntry( p, 0, des, dat, l, CatalogEntry::E_CATALOG ) );
    logList.append( ce );
    return ce;
}

/** add an entry to the tree of an existing catalogue */
CatTreeItem* CatalogHandler::
addEntry( CatTreeItem* parent, QString p, int s, QString des, QDate* dat, QStringList* l, int entry_type )
{
  CatTreeItem *ce = new CatTreeItem( parent,
    new CatalogEntry( p, s, des, dat, l, entry_type ) );
  return ce;
}

/** Reads the directory passed as argument and gets all needed infos */
bool CatalogHandler::scanDir( QDir dir, CatTreeItem *ce )
{
  dir.setSorting( QDir::DirsFirst | QDir::IgnoreCase );
  if( !dir.isReadable() )
    return true;
  const QFileInfoList * fil = dir.entryInfoList( QDir::All | QDir::Hidden );
  QFileInfoListIterator it( *fil );
  QFileInfo * fi;
  bool flag = false;
  while( (fi=it.current()) != 0 )
  {
    ++it;
    prgDlg->progressBar()->advance( 1 );
    if( !fi->isReadable() )
    {
      flag = true;
      continue;
    }
    QDate *dat = new QDate( fi->lastModified().date() );
    QString path = fi->absFilePath();
    QString name = path.right( path.length() - path.findRev( '/' ) - 1 );
    CatTreeItem *ce2;
    if( fi->fileName() == "." || fi->fileName() == "..")
      continue;
    if( fi->isDir() )
    {
      QDir dir2( path );
      ce2 = addEntry( ce, name, fi->size(), "", dat, catl, CatalogEntry::E_DIR );
      flag = scanDir( dir2, ce2 );
    }
    else if( fi->isSymLink() )
      ce2 = addEntry( ce, name, fi->size(), "", dat, catl, CatalogEntry::E_LINK );
    else if( fi->isFile() )
      ce2 = addEntry( ce, name, fi->size(), "", dat, catl, CatalogEntry::E_FILE );
    ce2->entry->SetIcon( KMimeType::iconForURL( path ) );
  }
  return flag;
}

/** Quickly scan mount point counting total number of files */
void CatalogHandler::totalEntries( QDir dir )
{
  prgDlg->progressBar()->advance( 1 );
  if( !dir.isReadable() )
    return;
  totEntries += dir.count();
  QStringList list( dir.entryList( QDir::Dirs | QDir::Hidden ) );
  for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
    if( *it == "." || *it == "..")
      continue;
    totalEntries( QDir( dir.absPath() + "/" + *it ) );
  }
}

void CatalogHandler::removeCatalog( CatTreeItem *cti )
{
  logList.remove( cti );
  delete cti;
}

void CatalogHandler::clear()
{
  QValueListIterator<CatTreeItem *> it = logList.begin();
  while ( it != logList.end() )
    delete *it++;
  logList.clear();
}

/** No descriptions */
void CatalogHandler::removeCategory( QString str )
{
  QValueListIterator<CatTreeItem *> it = logList.begin();
  while ( it != logList.end() )
    removeCategory( *it++, str );
}

/** No descriptions */
void CatalogHandler::removeCategory( CatTreeItem *ti, QString str )
{
  CatTreeItem *item = ti->child;
  while( item )
  {
    item->entry->delCat( str );
    if( item->child )
       removeCategory( item->child, str );
    item = item->next;
  }
}
