/***************************************************************************
                          catalogview.h  -  description
                             -------------------
    begin                : Fri Mar 28 2003
    copyright            : (C) 2003 by Stefano Salvador
    email                : salva_ste@tin.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CATALOGVIEW_H
#define CATALOGVIEW_H

#include <qwidget.h>

// Some declarations to speed up compilation
class QListViewItem;
class QPushButton;
class QSplitter;

class KListView;
class eKListViewItem;

class KatalogDoc;
class CategoryHandler;
class CatalogHandler;
class CatalogEntry;
class CatTreeItem;
class MidView;

/**This is the widget that handles the visualization of catalogues
  *@author Stefano Salvador
  */

class CatalogView : public QWidget  {
   Q_OBJECT
public:
  /** Public constructor */
  CatalogView( QWidget *parent = 0, const char *name = 0, KatalogDoc *pDoc = 0 );

  /** Public destructor */
  ~CatalogView();
  /** No descriptions */
  void addChildren( eKListViewItem * );
  /** No descriptions */
  void RefreshRView();

  /** No descriptions */
  void setCategories( CatTreeItem *ti = 0, QStringList *cl = 0, bool propagate = false );

public slots: // Public slots
  /** Used to manage the left dialog */
  void slotDone();

  /** Show/Hide Left Add Dialog */
  void slotToggledAdd( bool on );

  /** Show/Hide Left Edit Dialog */
  void slotToggledLEdit( bool on );

  /** Show/Hide Right Edit Dialog */
  void slotToggledREdit( bool on );

  /** Remove the selected entry */
  void slotLDelete();

  /**  */
  void slotLItemSelected( QListViewItem * );

  /**  */
  void slotRItemSelected( QListViewItem * );
  /** No descriptions */
  void slotRDelete();
  /** No descriptions */
  void RefreshLView();

signals: // Signals
  /** Emited when an entry changes */
  void catalogChanged();

private:
  KatalogDoc *doc;

  /** Set the contents of a CatalogEntry in the views */
  void setEntry( eKListViewItem *lv );

  /** Sets the contents of the middle view */
  void initMid();
  /** No descriptions */
  void RefreshChildren( eKListViewItem *lvi );

  /** Pointer to the master list view */
  KListView *lvM;

  /** The detailed view */
  KListView *lvD;

  /** "smart" left dialog  */
  MidView *lmid;

  /** "smart" right dialog  */
  MidView *rmid;

  /** "smart" add button in master view */
  QPushButton *ladd;

  /** "smart" edit button in master view */
  QPushButton *ledit;

  /** "smart" edit button in detailed view */
  QPushButton *redit;

  /** Pointer to current selected Entry in Master View */
  CatTreeItem *treeItemP;

  /** Pointer to current selected Entry in Detailed View */
  CatTreeItem *treeItemC;

  /** Pointer to current selected Item in Master View */
  eKListViewItem *lvMI;

  int midlg;

  enum MidDlg
  {
    LAdd = 0,
    LEdit = 1,
    REdit = 2
  };
};

#endif
