/***************************************************************************
                          searchview.cpp  -  description
                             -------------------
    begin                : Fri Mar 28 2003
    copyright            : (C) 2003 by Stefano Salvador
    email                : salva_ste@tin.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include "qlayout.h"
#include "qpushbutton.h"
#include "qcombobox.h"
#include "qlabel.h"
#include "qlineedit.h"
#include "qframe.h"
#include "qcheckbox.h"
#include "qlistbox.h"
#include "qvalidator.h"
#include "qvaluelist.h"

// include files for KDE
#include "klistview.h"
#include "kdatewidget.h"
#include "klocale.h"
#include "kmimetype.h"
#include "kiconloader.h"

// application specific includes
#include "searchview.h"
#include "category.h"
#include "categoryhandler.h"
#include "catalogentry.h"
#include "cataloghandler.h"
#include "cattreeitem.h"
#include "katalogdoc.h"

SearchView::SearchView(QWidget *parent, const char *name, KatalogDoc *pDoc ):
QWidget( parent , name )
{
  // Initialize private members
  doc = pDoc;

  QVBoxLayout *vbox = new QVBoxLayout( this );

  // Top
  QHBoxLayout *hbox = new QHBoxLayout( this );
  // Top::Left
  QFrame *left = new QFrame( this );
  left->setFrameStyle( QFrame::Panel | QFrame::Raised );
  QGridLayout *gridl = new QGridLayout( left, 4, 3, 10, 10 );
  // Top::Left::Row1
  eName = new QLineEdit( "", left );
  QRegExpValidator *rv = new QRegExpValidator( 0 );
  eName->setValidator( rv );
  QLabel* lName = new QLabel( eName, i18n("&Filename:"), left );
  gridl->addWidget( lName, 0, 0 );
  gridl->addWidget( eName, 0, 1 );
  // Top::Left::Row2
  eCase = new QCheckBox( i18n("Match case"), left );
  gridl->addWidget( eCase, 1, 1 );
  // Top::Left::Row3
  eCatalog = new QComboBox( left );
  QLabel* lCatalog = new QLabel( eCatalog, i18n("&Catalogue:"), left );
  gridl->addWidget( lCatalog, 2, 0 );
  gridl->addWidget( eCatalog, 2, 1 );
  // Top::Left::Row4
  QHBoxLayout *hpb = new QHBoxLayout( this, 6 );
  search = new QPushButton( i18n("Search"), this  );
  stop   = new QPushButton( i18n("Stop"), this );
  more   = new QPushButton( i18n("More >>"), this );
  hpb->addStretch( 1 );
  hpb->addWidget( search );
  hpb->addWidget( stop );
  hpb->addWidget( more );
  gridl->addLayout( hpb, 3, 1 );
  hbox->addWidget( left );

  // Top::Right
  right = new QFrame( this );
  right->setFrameStyle(QFrame::Panel | QFrame::Raised );
  QGridLayout *gridr = new QGridLayout( right, 4, 4, 10, 10 );
  // Top::Right::Row1
  eSize = new QLineEdit( "", right );
  QIntValidator *iv = new QIntValidator( 0 );
  eSize->setValidator( iv );
  mSize = new QComboBox( right );
  mSize->insertItem( i18n("At least"), -1 );
  mSize->insertItem( i18n("Not more than"), -1 );
  mSize->insertItem( i18n("Exactly"), -1 );
  dSize = new QComboBox( right );
  dSize->insertItem( i18n("Byte"), -1 );
  dSize->insertItem( i18n("KB"), -1 );
  dSize->insertItem( i18n("MB"), -1 );
  QLabel* lSize = new QLabel( eSize, i18n("&Size:"), right );
  gridr->addWidget( lSize, 0, 0 );
  gridr->addWidget( mSize, 0, 1 );
  gridr->addWidget( eSize, 0, 2 );
  gridr->addWidget( dSize, 0, 3 );
  // Top::Right::Row2
  mDate = new QComboBox( right );
  mDate->insertItem( i18n("Before"), -1 );
  mDate->insertItem( i18n("After"), -1 );
  mDate->insertItem( i18n("Exactly"), -1 );
  eDate = new KDateWidget( QDate::currentDate(), right );
  QLabel* lDate = new QLabel( eDate, i18n("&Date:"), right );
  gridr->addWidget( lDate, 1, 0 );
  gridr->addWidget( mDate, 1, 1 );
  gridr->addMultiCellWidget( eDate, 1, 1, 2, 3 );
  // Top::Right::Row3
  eCategory = new QListBox( right );
  eCategory->setMaximumHeight( 60 );
  eCategory->setSelectionMode( QListBox::Multi );
  QLabel* lCategory = new QLabel( eCategory, i18n("&Category:"), right );
  gridr->addWidget( lCategory, 2, 0 );
  gridr->addMultiCellWidget( eCategory, 2, 3, 1, 3 );
  hbox->addWidget( right );

  // Bottom
  view = new KListView( this );
  view->addColumn( i18n( "Filename" ) );
  view->addColumn( i18n( "Catalog" ) );
  view->addColumn( i18n( "Size" ) );
  view->addColumn( i18n( "Date" ) );
  view->addColumn( i18n( "Category" ) );
  view->addColumn( i18n( "Description" ) );
  view->setFullWidth();

  vbox->addLayout( hbox );
  vbox->addWidget( view );

  connect( more, SIGNAL( clicked() ), this, SLOT( toggleRight() ) );
  connect( search, SIGNAL( clicked() ), this, SLOT( Search() ) );
  connect( stop, SIGNAL( clicked() ), this, SLOT( Stop() ) );
  opt = false;
  fstop = false;
  right->hide();
  stop->setEnabled( false );
  initView();
}

SearchView::~SearchView()
{
}

void SearchView::initView()
{
  // Refresh Catalog ComboBox
  eCatalog->clear();

  QValueListIterator<CatTreeItem *> it = doc->logHand->logList.begin();
  eCatalog->insertItem( i18n("All Catalogs") );
  while ( it != doc->logHand->logList.end() )
  {
    eCatalog->insertItem( (*it)->entry->Name() );
    it++;
  }

  // Refresh Category ComboBox
  eCategory->clear();
  QStringList * cl = doc->oryHand->categoriesList();
  if( cl )
    eCategory->insertStringList( *cl );
}

void SearchView::toggleRight()
{
  opt = !opt;
  if( opt )
  {
    more->setText( i18n("Less <<") );
    right->show();
  }
  else
  {
    more->setText( i18n("More >>") );
    right->hide();
  }
}

void SearchView::Search()
{
  view->clear();
  stop->setEnabled( true );
  more->setEnabled( false );
  fstop = false;
  QValueListIterator<CatTreeItem *> it = doc->logHand->logList.begin();
  if( eCatalog->currentItem() == 0 )
  {
    while ( it != doc->logHand->logList.end() )
      SearchBranch( *it++ );
  }
  else
  {
    while ( it != doc->logHand->logList.end() )
    {
      if( (*it)->entry->Name() == eCatalog->currentText() )
        SearchBranch( *it );
      it++;
    }
  }

  stop->setEnabled( false );
  more->setEnabled( true );
}

void SearchView::Stop()
{
  fstop = true;
}

/** Set the contents of a CatalogEntry */
void SearchView::setEntry( CatalogEntry *ce )
{
  KListViewItem *e = new KListViewItem( view );
  e->setText( 0, ce->Name() );
  e->setText( 1, ce->MyCatalog() );
  e->setText( 2, QString::number( ce->Size() ) );
  e->setText( 3, ( ce->Date() )->toString( "dd.MM.yyyy" ) );

  QStringList *l = ce->CatList();
  if( l != 0)
    e->setText( 4, l->join( "::" ) );

  e->setText( 5, ce->Description() );
  e->setPixmap(
    0,
    KGlobal::iconLoader()->loadIcon( ce->Icon(), KIcon::Small ) );

}

/** Follows a branch searching a match */
void SearchView::SearchBranch( CatTreeItem *ti )
{
  bool flag = false;
  CatTreeItem *item = ti;
  while( item )
  {
    // Set the entry
    CatalogEntry *e = item->entry;

    // Check if a recursion is needed
    if( item->child )
      SearchBranch( item->child );

    // Increment item
    item = item->next;

    // Check for user interrupt
    if( fstop )
      break;

    // Now start the search ...
    // Check name
    if( e->Name().find( QRegExp( eName->text(), eCase->isChecked() ) ) == -1 )
      continue;

    // If Fast Search Mode stop here
    if( !opt )
    {
      setEntry( e );
      continue;
    }

    // Check size
    int s = eSize->text().toInt();
    switch( dSize->currentItem() )
    {
      case 0:
        break;
      case 1:
        s *= 1024;
        break;
      case 2:
      default:
        s *= 1048576;
        break;
    }
    if( s )
    {
      flag = false;
      switch( mSize->currentItem() )
      {
      case 0:
        if( e->Size() < s )  // At Least
          flag = true;
        break;
      case 1:
        if( e->Size() >= s ) // Not More Than
          flag = true;
        break;
      case 2:
      default:
        if( e->Size() != s ) // Exactly
          flag = true;
        break;
      }
      if( flag )
        continue;
    }

    // Check date
    flag = false;
    QDate dateComp( eDate->date() );
    QDate dateEntry( *(e->Date()) );
    switch( mDate->currentItem() )
    {
    case 0:
      if( dateEntry > dateComp )   // Before
        flag = true;
      break;
    case 1:
      if( dateEntry < dateComp )   // After
        flag = true;
      break;
    case 2:
    default:
      if( dateEntry != dateComp )  // Exactly
        flag = true;
      break;
    }
      if( flag )
        continue;

    // Check Categories
    flag = false;
    QStringList * cl = e->CatList();
    for( int c = 0; c < (int)eCategory->count(); c++ )
    {
      if( !cl )
      {
        flag = true;
        break;
      }
      if( eCategory->item( c )->isSelected() )
      {
        if( cl->find( eCategory->text( c ) ) == cl->end() )
        {
          flag = true;
          break;
        }
      }
    }
    if( flag )
      continue;

    // OK, I found a corrispondence
    setEntry( e );
  }
}
